// file name:  camac.c
// Date: Mar 6, 2003
// Edited by:  E.Inoue
//
// Parallel camac
// packet format of basic camac opeation
// TX
// +---------+-------+-------+-------+-------+---------------------------+
// |   cmd   |   N   |   A   |   F   |       |       24-bit DATA         |
// +---------+-------+-------+-------+-------+---------------------------+
//
// RX
// +---------+-------+-------+-------+-------+---------------------------+
// |  rply   |   N   |   A   |   F   |  ST   |       24-bit DATA         |
// +---------+-------+-------+-------+-------+---------------------------+

#include    <stdio.h>
#include    <jni.h>
#include    <sys/io.h>
#include    "camac.h"

#define BASE 0xE800
#define TX_DATA1REG BASE
#define TX_DATA2REG (BASE+4)
/* #define TX_STATUSREG (BASE+0x10) */
#define TX_STATUSREG (BASE+0x1c)
#define TXSTATUS_BUSY 0x10000
#define TXSTATUS_TIMEOUT 0x20000
#define RX_DATA1REG (BASE+0x20)
#define RX_DATA2REG (BASE+0x24)
/* #define RX_FIFOREG  (BASE+0x30) */
#define RX_FIFOREG  (BASE+0x3c)
/*
#define CMD_F16_A0_N3 0xE0030010
#define CMD_F0_A0_N3 0xE0030000
*/
#define CMD_F16_A0_N3 0xE0040010
#define CMD_F0_A0_N3 0xE0040000

unsigned long long int value;
unsigned int camac;


JNIEXPORT jint JNICALL 
Java_camac_cmdArray(JNIEnv *env, jobject obj, jintArray r_cmd, jintArray r_dat, jint length)
{
	int	i, sum = 0;
	jsize	len = (*env)->GetArrayLength(env, r_cmd);
	jint	*c_cmd = (*env)->GetIntArrayElements(env, r_cmd, 0);

	jsize	len2 = (*env)->GetArrayLength(env, r_dat);
	jint	*c_dat = (*env)->GetIntArrayElements(env, r_dat, 0);

	int	loop = length;
	int	ii, j, data, cmd, value;

	printf ("\n   JNI, C side: Received CAMAC command: ");
	printf ("array length = %d\n", length); 
	for (i=0; i < length +1; i++) {
	   printf("      c_cmd[%d] = 0x%x, c_dat[%d] = 0x%x\n", i, c_cmd[i], i, c_dat[i]);
	}

	if( iopl(3) ){
           printf("can not change the privilege level\n");
           exit(0);
	}

	printf("\n   JNI, C side: CAMAC access\n");
	for( i = 0; i < loop; i++) {
	   switch (c_cmd[i] & 0x00000018) {

//  CAMAC Write
		case 0x00000010 :

//  ǽ֤ˤʤޤԤ
			value = inl(TX_STATUSREG);
			while (value & TXSTATUS_BUSY) {
			   if (value & TXSTATUS_TIMEOUT) break;
			   value = inl(TX_STATUSREG);
			} 

//  camac writeޥ&ǡ򥻥åȤơPCI쥸˽񤭹
			data = c_dat[i] & 0x00ffffff;
			outl(data, TX_DATA1REG);
			outl(CMD_F16_A0_N3, TX_DATA2REG);
			printf("      camac write data:  data = 0x%x\n", data);

//  camacɤ߽Фǽ֤ˤʤޤԤ
			value = 0;
			do {
			   value = inl(RX_FIFOREG);
			   // for PCI ROM of new version 
			   //  value = value >> 24;
			} while( value < 2 );

//  camacμѥ쥸ɤ camacα
			data = inl(RX_DATA1REG);
			cmd = inl(RX_DATA2REG);
			printf("      camac write command reply:  cmd = 0x%x : data = %x\n", cmd, data);
			c_cmd[i] = cmd;
			c_dat[i] = data;
			break;


//  CAMAC Read
		case 0x00000000 :

//  ǽ֤ˤʤޤԤ
			value = inl(TX_STATUSREG);
			while (value & TXSTATUS_BUSY) {
			if (value & TXSTATUS_TIMEOUT) break;
			   value = inl(TX_STATUSREG);
			}

//  camac readޥ&ǡ򥻥åȤơPCI쥸˽񤭹
			outl(CMD_F0_A0_N3, TX_DATA2REG);

//  camacɤ߽Фǽ֤ˤʤޤԤ
			value = 0;
			do {
			   value = inl(RX_FIFOREG);
			   // for PCI ROM of new version 
			   //  value = value >> 24;
			} while( value < 2);

//  camacμѥ쥸ɤ camacα
			data = inl(RX_DATA1REG);
			cmd = inl(RX_DATA2REG);
			data = 0xffffff & data;
			ii = 0xffffff & c_dat[i];
			printf("      camac read command reply:  cmd = 0x%x : data = 0x%x\n", cmd, data);
			c_cmd[i] = cmd;
			c_dat[i] = data;
			break;

//  Default
		default :
			break;
	   }
	}

	(*env)->ReleaseIntArrayElements(env, r_cmd, c_cmd, 0);
	(*env)->ReleaseIntArrayElements(env, r_dat, c_dat, 0);
	return sum;
}
